# -----------------------------------------------------------------------------
sub sqlTrends
# -----------------------------------------------------------------------------
# Look for exceptions in trends. Uses perf data from prior iterations
# -----------------------------------------------------------------------------
# Changes:
#   2014/07/30 BB: Changed $tr2_t_io from 100 to 200
# -----------------------------------------------------------------------------
{
  ($thishost) = @_;

  if ($up[$thishost] ne "Y")
  {
    &logit("(Down/Inhibited)",$thishost);
    return 1;
  }

  if (&check_inhib($thishost) eq "Y")
  {
    $up[$thishost] = "I";
    return;
  }

  #---- Vars
  my $this_line = ""; 
  my @Data;

  my $debug_tr = 0;
  my $trend_event_count = 0; 
  my $trend_rows        = ""; 
  my $trend_rows_other  = ""; 
  my $tr_recent_db_rsvd_mb = -1; 
  my $tr_hist_db_rsvd_mb   = -1; 
  my %tr_recent_avg = (); 
  my $tr_color = ""; 
  my $tr2_color = ""; 

  $tr_recent_days = 3;
  $tr_hist_days   = 180;
  $tr_threshold   = 150;    #---- Temporary

  #-----------------------------------------------------------------------------
  #---- Recent readings which will be used later
  #-----------------------------------------------------------------------------

  $sql  = "select
        category,
        parm,
        count(*),
        avg(value)
      from
        dbamon.metrics_detail
      where
        instance = '$titleraw[$thishost]'
        and dbamonmode = '$mode'
        and ts >= dateadd(dd,-$tr_recent_days,getdate())
      group by
        category,
        parm
      order by 1, 2";

  $sth = $dbh->prepare($sql) || &error("DB_prepare");
  $sth->execute || &error("DB_execute");
  while (@Data = $sth->fetchrow_array)
  {
    $tr_cat    = shift(@Data); 
    $tr_parm   = shift(@Data); 
    $tr_cnt    = shift(@Data); 
    $tr_avgval = shift(@Data); 
    ##print "DEBUG tr_cat=$tr_cat tr_parm=$tr_parm tr_cnt=$tr_cnt tr_avgval=$tr_avgval\n"; 
    $tr_recent_avg{"$tr_cat/$tr_parm"} = $tr_avgval; 
  }

  #-----------------------------------------------------------------------------
  #- DBSize - recent vs. historical
  #-----------------------------------------------------------------------------

  #---- Add the values for all databases into total var
  foreach $tr_key (%tr_recent_avg) 
  {
    if ($tr_key =~ /^SPC_DB_RESERVED_MB/) { $tr_recent_db_rsvd_mb += sprintf("%.2f", $tr_recent_avg{$tr_key}); }
  }
  print "DEBUG - $titleraw[$thishost] - tr_recent_db_rsvd_mb=$tr_recent_db_rsvd_mb\n" if $tr_debug; 

  #---- Calc AVG of daily AVG historical value for last N days
  $tr_hist_db_rsvd_mb = &sqlTrendsHistQ('SPC_DB_RESERVED_MB');   
  print "DEBUG - $titleraw[$thishost] - tr_hist_db_rsvd_mb=$tr_hist_db_rsvd_mb\n" if $tr_debug; 

  my $tr_db_rsvd_mb_recent_vs_hist_pct = sprintf("%.2f", $tr_recent_db_rsvd_mb / $tr_hist_db_rsvd_mb ) * 100 if $tr_hist_db_rsvd_mb > 0; 
  print "DEBUG - $titleraw[$thishost] - tr_db_rsvd_mb_recent_vs_hist=$tr_db_rsvd_mb_recent_vs_hist\n" if $tr_debug; 

  if ($tr_recent_db_rsvd_mb > 1000 && $tr_db_rsvd_mb_recent_vs_hist_pct >= $tr_threshold) 
  {
    $tr_color = "pink"; 
    $trend_event_count++; 
    #---- Event here
    open(TMPFL2, ">$tmpfl2") || &error_msg("Cannot open output $tmpfl2 - $!");
    print TMPFL2 "
Metric:               SPC_DB_RESERVED_MB
Recent_Reading:       $tr_recent_db_rsvd_mb  (AVG Reading over the last $tr_recent_days days) 
Historical_Reading:   $tr_hist_db_rsvd_mb  (AVG Reading over the last $tr_hist_days days) 
Recent_vs_Historical: ${tr_db_rsvd_mb_recent_vs_hist_pct}%  
    ";
    close(TMPFL2);
    ##"sqlTrends: Metric SPC_DB_RESERVED_MB - Recent Reading is $tr_db_rsvd_mb_recent_vs_hist_pct Percent versus Historical Reading (T=$tr_threshold)",
    $ev_cat = "Space";
    &event("U",$thishost,
      "sqlTrends: Metric SPC_DB_RESERVED_MB - Recent Reading is Higher than Historical Reading (Threshold=$tr_threshold Percent)",
      "Trend-SPC_DB_RESERVED_MB","DBA950", $tmpfl2);
  }
  else
  {
    $tr_color = "lime"; 
  }

  $trend_rows .= "<tr bgcolor=$table_rowcolor>
    <td nowrap> $f_s <b>Trend_DBRsvdSpaceMB_Pct </b>
    <td align=right bgcolor=$tr_color>$f_s <b> $tr_db_rsvd_mb_recent_vs_hist_pct </b>
    <td> $f_s The recent (last $tr_recent_days days) reading of SPC_DB_RESERVED_MB (for all DBs) vs. the historical average (last $tr_hist_days) days) as a Percent 
  "; 

  &metricsInsert("sqlTrends()", $thishost, "TREND_DBRsvdSpaceMB_Pct", ".", $tr_db_rsvd_mb_recent_vs_hist_pct);

  &logit("  Trend_DBRsvdSpaceMB_Pct: $tr_recent_db_rsvd_mb / $tr_hist_db_rsvd_mb = $tr_db_rsvd_mb_recent_vs_hist_pct",$thishost);

  #-----------------------------------------------------------------------------
  #- 5.49.B 2014/05/09: New check for multiple PVSCSI adapters on busy VMware SQL boxes. 
  #-----------------------------------------------------------------------------

  my $tr2_t_io     = 200; 
  my $tr2_days_io  = 30; 
  my $tr2_result   = ""; 

  #---- Gate 1 - Historical I/O rate
  $sql  = "select
	    AVG(AVG)
    FROM    dbamon.dbamon.METRICS_DAILY
    WHERE   ts > DATEADD(dd, -$tr2_days_io, GETDATE())
	    AND dbamonmode = 'prod'
	    AND category = 'PERF_SYS_DISKTOTTRANSFERSPERSEC'
	    AND parm = '.'
	    AND instance = '$titleraw[$thishost]'
  "; 

  $sth = $dbh->prepare($sql) || &error("DB_prepare");
  $sth->execute || &error("DB_execute");

  $tr2_g1 = -1; 
  ##print "DEBUG_T2 sql=$sql"; 

  while (@Data = $sth->fetchrow_array)
  {
    $tr2_g1 = shift(@Data); 
  }
  $tr2_g1 = -1 if $tr2_g1 == 0; 

  #---- Gate 2 - virtual server
  $tr2_g2 = $server_type[$thishost]; 

  #---- Gate 3 - Number of PVSCSI controllers 
  $tr2_g3 = $perf_readings[$thishost]{"SYS_SCSI_Cont_PVSCSI"}; 
  ##print "tr2_g3=($tr2_g3)\n"; 

  #---- For debugging
  #foreach $z (sort keys $perf_readings[$thishost] )
  #{
  #  print "z=$z\n"; 
  #}

  #---- Pass/fail logic
  
  if ($tr2_g2 !~ /^VM/) 
  {
    $tr2_color = "white"; 
    $tr2_result .= "(Non-VMware server)"; 
  }
  else
  {
    if ($tr2_g1 < $tr2_t_io) 
    {
      $tr2_color = "white"; 
      $tr2_result .= "(I/O Rate < $tr2_t_io)";  
    } 
    else   #---- We have a vmware server with a qualifying I/O rate - so we DO want to check PVSCSI count
    {
      if ($tr2_g3 > 1) 
      {
	$tr2_color = "lime"; 
	$tr2_result .= "(PVSCSI Cont. count GT 1)";  
      }
      else
      {
	$tr2_color = "pink"; 
	$tr2_result .= "(PVSCSI Cont. count LT 2)";  
        #---- Create new EVENT here
	open(TMPFL2, ">$tmpfl2") || &error_msg("Cannot open output $tmpfl2 - $!");
	print TMPFL2 "
G1 - Historical_I/O_Rate: $tr2_g1  (Average I/O Per Second Rate over the last $tr2_days_io days - Threshold: $tr2_t_io)
G2 - Server_Type:         $tr2_g2  (Physical or VMware) 
G3 - PVSCSI_Cont_Count:   $tr2_g3  (Should be GT 1 for a busy VMware server)  
	";
	close(TMPFL2);
        my $tr2_evstring = "VMware SQL Server With High Historical I/O Rate Should Have Multiple PVSCSI Controllers"; 
        &event("P", $thishost, $tr2_evstring, "Perf-PVSCSI", "DBA960", $tmpfl2);
        $trend_event_count++; 
      } 
    }
  }

  $trend_rows_other = "<tr bgcolor=$tr2_color>
  <td>$f_s <b>VMware PVSCSI Check</b>
  <td>$f_s <b>Gate1=($tr2_g1) Gate2=($tr2_g2) Gate3=($tr2_g3) Result=$tr2_result </b>
  <td>$f_s The goal of this event is that we want to have multiple PVSCSI Controllers defined on all Busy 
	   (at least $tr2_t_io average I/Os PS over the last $tr2_days_io days) VMware servers
  "; 

  #-----------------------------------------------------------------------------
  #- 5.53.A 2015/06/12 - New SA Sessions HWM (Last 24 hours) Check
  #-----------------------------------------------------------------------------

  my $tr_sa_days= 1; 
  my $tr_sa_debug = 0; 

  #---- Find max number of concurrent SA logins over the last $tr_sa_days days
  $sql  = "
  select max( value )
  from dbamon.METRICS_DETAIL 
  where instance = '$titleraw[$thishost]' and dbamonmode = '$mode' 
  and category = 'PERF_SQL_SESSIONS_SA' and parm = '.' and ts > dateadd(dd, -$tr_sa_days, getdate()) 
  "; 

  $sth = $dbh->prepare($sql) || &error("DB_prepare");
  $sth->execute || &error("DB_execute");
  print "DEBUG_SA sql=($sql)\n" if $tr_sa_debug; 

  while (@Data = $sth->fetchrow_array)
  {
    $tr_sa_maxsalogins = shift(@Data); 
  }

  print "DEBUG_SA tr_sa_maxsalogins=($tr_sa_maxsalogins)\n" if $tr_sa_debug; 

  if ($tr_sa_maxsalogins > $t_sa_max_logins[$thishost]) 
  {
    open(TMPFL2, ">$tmpfl2") || &error_msg("Cannot open output $tmpfl2 - $!");
    print TMPFL2 "It is an MSSQL Best Practice to use the SA login only when absolutely necessary and 
only by authorized (DBA) personnel. An Excessive number of concurrent SA logins may 
indicate that an application is connecting to MSSQL using the SA login. If this is the
case, you should create a Window-autheticated MSSQL Login which is used exclusively by 
only one application. If this is not possible, create an MSSQL Login that is used by 
only one application.

Max Concurrent SA Logins:  $tr_sa_maxsalogins  (This instance, last $tr_sa_days day(s)) 
DBAmon DBC Threshold:      $t_sa_max_logins[$thishost]  (DBC Parm: T_SA_Max_Logins:)
    ";
    close(TMPFL2);
    my $tr_sa_evstring = "MSSQL Login=SA Max Concurrent Logins: $tr_sa_maxsalogins (Last $tr_sa_days Day(s)) Exceeds Threshold=$t_sa_max_logins[$thishost]"; 
    &event("S", $thishost, $tr_sa_evstring, "Sec-SALogins", "DBA961", $tmpfl2);
    $trend_event_count++; 
  }

  #-----------------------------------------------------------------------------
  #- Finish
  #-----------------------------------------------------------------------------

  # &event($sqlperf_sev_c,$thishost, $zshort_c,"Perf-${this_metric}","DBA852", $tmpfl2);

  &logDetails($thishost,"<br><a name='sqlTrends'> <h3>MSSQL Trend Data </h3>");

  &logDetails($thishost,"<table border=0 bgcolor=$table_bgcolor>");
  &logDetails($thishost,"<tr bgcolor=$table_headbgcolor>
    <th>$f_s Trend Metric 
    <th>$f_s Current Reading<br>vs. Threshold=$tr_threshold
    <th>$f_s Description 
  ");
  &logDetails($thishost,$trend_rows); 
  &logDetails($thishost,"</table><hr>");

  $trend_rows_other = "<tr bgcolor=white><td colspan=3>$f_s <b>N/A</b>" if $trend_rows_other eq ""; 

  &logDetails($thishost,"<table border=0 bgcolor=$table_bgcolor>");
  &logDetails($thishost,"<tr bgcolor=$table_headbgcolor>
    <th>$f_s Trend Metric 
    <th>$f_s Current Reading
    <th>$f_s Description 
  ");
  &logDetails($thishost,$trend_rows_other); 
  &logDetails($thishost,"</table><hr>");

  &logit("OK - Total Trend Events this iteration: $trend_event_count",$thishost);

  return; 
}

# -----------------------------------------------------------------------------
sub sqlTrendsHistQ
# -----------------------------------------------------------------------------
{
  my $st_categ = $_[0]; 
  my $st_return = -999999;  

  #---- Calc AVG of daily AVG historical value for last N days
  $sql  = "select
    avg(il.sum_avg)
  from
    (select
	CONVERT(VARCHAR(10), ts, 101)
	zdate,
	sum(avg) as sum_avg
      from
	dbamon.metrics_daily
      where
	instance = '$titleraw[$thishost]'
	and dbamonmode = '$mode'
	and ts >= dateadd(dd,-$tr_hist_days,getdate())
	and category = '$st_categ'
      group by
	CONVERT(VARCHAR(10), ts, 101)) il ";
  ## print "DEBUG - sql=$sql\n"; 
  $sth = $dbh->prepare($sql) || &error("DB_prepare");
  $sth->execute || &error("DB_execute");

  while (@Data = $sth->fetchrow_array)
  {
    $st_return = shift(@Data);   
  }

  $st_return = sprintf("%.2f", $st_return); 
  return $st_return; 
}

# -----------------------------------------------------------------------------
1;
